/*=====================================================================
  File:      SecureConversationService.cs

  Summary:   This is a sample which allows the user to send an encrypted message
             to a Web service using policy and secure conversation.
             The policy uses the UsernameForCertificate policy assertion
             to protect the RST/RSTR message exchange and the subsequent requests.
 
---------------------------------------------------------------------
  This file is part of the Web Services Enhancements 3.0 for Microsoft .NET Samples.

  Copyright (C) Microsoft Corporation.  All rights reserved.

This source code is intended only as a supplement to Microsoft
Development Tools and/or on-line documentation.  See these other
materials for detailed information regarding Microsoft code samples.

This sample is designed to demonstrate WSE features and is not intended 
for production use. Code and policy for a production application must be 
developed to meet the specific data and security requirements of the 
application.

THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
PARTICULAR PURPOSE.
=====================================================================*/

using System;
using System.Collections.Generic;
using System.Web;
using System.Web.Services;
using System.Web.Services.Protocols;
using System.Xml.Serialization;

using Microsoft.Web.Services3;
using Microsoft.Web.Services3.QuickStart;

[WebService(Namespace = "http://stockservice.contoso.com/wse/samples/2005/10")]
[WebServiceBinding(ConformsTo = WsiProfiles.BasicProfile1_1)]
[Policy("ServerPolicy")]
public class StockService : System.Web.Services.WebService
{
    public StockService()
    {
    }

    [WebMethod]
    [SoapDocumentMethod(ResponseElementName = "StockQuotes")]
    [return: XmlElement("StockQuote")]
    public List<StockQuote> StockQuoteRequest([XmlArray(IsNullable = true), XmlArrayItem("Symbol", IsNullable = false)] string[] symbols)
    {
        List<StockQuote> quotes = new List<StockQuote>();

        foreach (String symbol in symbols)
        {
            StockQuote quote = new StockQuote();
            quote.Symbol = symbol;
            if (symbol == "FABRIKAM")
            {
                quote.Name = "Fabrikam, Inc.";
                quote.Last = 120.00;
                quote.PreviousChange = 5.5;
            }
            else
            {
                quote.Name = "Contoso Corp.";
                quote.Last = 50.07;
                quote.PreviousChange = 1.15;
            }
            quotes.Add(quote);
        }
        return quotes;
    } 
}
