using System;
using System.Collections.Generic;
using System.Text;

namespace PurpleMediaCenter.FinderServices
{
  /// <summary>
  /// For test purpose.
  /// </summary>
  public static class GlobalRepository
  {
    #region fields

    private static readonly MovieKeyedCollectionByTitle _movies = new MovieKeyedCollectionByTitle();
    private static readonly PersonKeyedCollectionByName _persons = new PersonKeyedCollectionByName();
    
    public static readonly Movie LeGoutDesAutres = new Movie("Le Got des autres", 112, new DateTime(2000, 3, 1));
    public static readonly Movie SousLeSable = new Movie("Sous le sable", 95, new DateTime(2001, 2, 7));
    public static readonly Movie PulpFiction = new Movie("Pulp fiction", 149, new DateTime(1994, 10, 26));
    public static readonly Movie SinCity = new Movie("Sin City", 149, new DateTime(1994, 10, 26));

    public static readonly Person QuentinTarantino = new Person("Quentin", "Tarantino", Genre.Male);
    public static readonly Person RobertRodriguez = new Person("Robert", "Rodriguez", Genre.Male);
    public static readonly Person AgneJaoui = new Person("Agns", "Jaoui", Genre.Female);
    public static readonly Person FrancoisOzon = new Person("Franois", "Ozon", Genre.Male);

    public static readonly Person JeanPierreBacri = new Person("Jean-Pierre", "Bacri", Genre.Male);
    public static readonly Person AnneAlvaro = new Person("Anne", "Alvaro", Genre.Female);
    public static readonly Person GerardLanvin = new Person("Grard", "Lanvin", Genre.Male);

    public static readonly Person JohnTravolta = new Person("John", "Travolta", Genre.Male);
    public static readonly Person SamuelLJackson = new Person("Samuel L.", "Jackson", Genre.Male);
    public static readonly Person UmaThurman = new Person("Uma", "Thurman", Genre.Female);
    public static readonly Person BruceWillis = new Person("Bruce", "Willis", Genre.Male);

    public static readonly Person BrunoCremer = new Person("Bruno", "Cremer", Genre.Male);
    public static readonly Person CharlotteRampling = new Person("Charlotte", "Rampling", Genre.Female);

    public static readonly Person MickeyRourke = new Person("Mickey", "Rourke", Genre.Male);
    public static readonly Person JessicaAlba = new Person("Jessica", "Alba", Genre.Female);

    public static readonly Movie[] TopMovies = null;
    public static readonly Person[] TopActors = null;

    #endregion
    #region ctors

    static GlobalRepository()
    {
      LeGoutDesAutres.AddDirector(AgneJaoui);
      LeGoutDesAutres.AddActor(AgneJaoui);
      LeGoutDesAutres.AddActor(JeanPierreBacri);
      LeGoutDesAutres.AddActor(AnneAlvaro);
      LeGoutDesAutres.AddActor(GerardLanvin);

      PulpFiction.AddDirector(QuentinTarantino);
      PulpFiction.AddActor(JohnTravolta);
      PulpFiction.AddActor(SamuelLJackson);
      PulpFiction.AddActor(UmaThurman);
      PulpFiction.AddActor(BruceWillis);

      SousLeSable.AddDirector(FrancoisOzon);
      SousLeSable.AddActor(CharlotteRampling);
      SousLeSable.AddActor(BrunoCremer);

      SinCity.AddDirector(QuentinTarantino);
      SinCity.AddDirector(RobertRodriguez);
      SinCity.AddActor(BruceWillis);
      SinCity.AddActor(MickeyRourke);
      SinCity.AddActor(JessicaAlba);

      _movies.Add(LeGoutDesAutres);
      _movies.Add(PulpFiction);
      _movies.Add(SousLeSable);
      _movies.Add(SinCity);

      _persons.Add(AgneJaoui);
      _persons.Add(JeanPierreBacri);
      _persons.Add(AnneAlvaro);
      _persons.Add(GerardLanvin);
      _persons.Add(QuentinTarantino);
      _persons.Add(RobertRodriguez);
      _persons.Add(JohnTravolta);
      _persons.Add(SamuelLJackson);
      _persons.Add(UmaThurman);
      _persons.Add(FrancoisOzon);
      _persons.Add(CharlotteRampling);
      _persons.Add(BrunoCremer);
      _persons.Add(BruceWillis);
      _persons.Add(MickeyRourke);
      _persons.Add(JessicaAlba);

      List<Movie> __movies = new List<Movie>();
      __movies.Add(PulpFiction);
      __movies.Add(SousLeSable);
      __movies.Add(SinCity);

      TopMovies = __movies.ToArray();

      List<Person> __persons = new List<Person>();
      __persons.Add(BrunoCremer);
      __persons.Add(BruceWillis);
      __persons.Add(MickeyRourke);
      __persons.Add(JessicaAlba);

      TopActors = __persons.ToArray();
    }

    #endregion
    #region methods

    public static Movie[] GetMoviesByActor(Person actor)
    {
      List<Movie> __movies = new List<Movie>();
      foreach (Movie movie in _movies)
      {
        if (movie.HasARole(actor))
        {
          __movies.Add(movie);
        }
      }
      return __movies.ToArray();
    }
    public static Movie[] GetMoviesByDirector(Person director)
    {
      List<Movie> __movies = new List<Movie>();
      foreach (Movie movie in _movies)
      {
        if (movie.WasDirectedBy(director))
        {
          __movies.Add(movie);
        }
      }
      return __movies.ToArray();
    }
    public static Movie[] GetMoviesByTitle(string title)
    {
      List<Movie> __movies = new List<Movie>();
      if (_movies.Contains(title))
      {
        __movies.Add(_movies[title]);
      }
      return __movies.ToArray();
    }
    public static Movie[] GetMoviesByDirectorFor(Person director, int[] years)
    {
      List<Movie> __movies = new List<Movie>();
      List<int> __years = new List<int>(years);

      foreach (Movie movie in _movies)
      {
        if (movie.WasDirectedBy(director) && __years.Contains(movie.ReleaseDate.Year))
        {
          __movies.Add(movie);
        }
      }
      return __movies.ToArray();
    }

    public static Person[] WereDirectedBy(Person director)
    {
      PersonKeyedCollectionByName __actors = new PersonKeyedCollectionByName();
      Movie[] __movies = GetMoviesByDirector(director);
      foreach (Movie movie in __movies)
      {
        foreach (Person actor in movie.Actors)
        {
          if (!__actors.Contains(actor))
          {
            __actors.Add(actor);
          }
        }
      }
      return __actors.ToArray();
    }

    public static void AddMovie(Movie movie)
    {
      _movies.Add(movie);
      AddActors(movie.Actors);
      AddDirectors(movie.Directors);
    }
    public static void AddActor(Person actor)
    {
        _persons.Add(actor);
    }
    public static void AddActors(Person[] actors)
    {
      foreach (Person person in actors)
      {
        AddActor(person);
      }
    }
    public static void AddDirector(Person director)
    {
      if (!_persons.Contains(director))
      {
        _persons.Add(director);
      }
    }
    public static void AddDirectors(Person[] directors)
    {
      foreach (Person person in directors)
      {
        AddActor(person);
      }
    }

    #endregion
  }
}
