using System;
using System.Collections.Generic;
using System.Text;

namespace PurpleMediaCenter
{
  [Serializable]
  public class Movie : IMovie
  {
    #region fields

    private string _title;
    private PersonKeyedCollectionByName _directors;
    private PersonKeyedCollectionByName _actors;
    private int _runtime;
    private DateTime _releaseDate;

    #endregion
    #region ctors
    
    public Movie()
    {
      _directors = new PersonKeyedCollectionByName();
      _actors = new PersonKeyedCollectionByName();
    }
    public Movie(string title, int runtime, DateTime releaseDate):this()
    {
      _title = title;
      _runtime = runtime;
      _releaseDate = releaseDate;
    }

    #endregion
    #region props

    public string Title
    {
      get { return _title; }
      set { _title = value; }
    }
    public Person[] Directors
    {
      get { return _directors.ToArray(); }
      set { _directors = new PersonKeyedCollectionByName(value); }
    }
    public Person[] Actors
    {
      get { return _actors.ToArray(); }
      set { _actors = new PersonKeyedCollectionByName(value); }
    }
    public int Runtime
    {
      get { return _runtime; }
      set { _runtime = value; }
    }
    public DateTime ReleaseDate
    {
      get { return _releaseDate; }
      set { _releaseDate = value; }
    }

    #endregion
    #region methods

    public void AddDirector(Person director)
    {
      _directors.Add(director);
    }
    public void AddActor(Person actor)
    {
      _actors.Add(actor);
    }
    public bool WasDirectedBy(Person director)
    {
      return _directors.Contains(PersonKeyedCollectionByName.GetKey(director));
    }
    public bool HasARole(Person actor)
    {
      return _actors.Contains(PersonKeyedCollectionByName.GetKey(actor));
    }

    #endregion
  }
}
