#region (c) 2002-2006 by thinktecture
// **********************************************************************
//
//  2002-2006 by thinktecture, Ingo Rammer and Christian Weyer. All rights reserved.
//
// This copy of thinktecture is licensed to you under the terms described in the
// License.txt file included in this distribution.
//
// Author: 
// Christian Weyer (christian.weyer@thinktecture.com)
//
// **********************************************************************
#endregion
#region (c) 2006 by netfxfactory.org
// **********************************************************************
//
//  2006 by netfxfactory.org. All rights reserved.
//
// This copy is licensed to you under the terms described in the
// License.txt file included in this distribution.
//
// Author: 
// pascal.recchia@netfxfactory.org
// anthony.guerot@netfxfactory.org
// **********************************************************************
#endregion

using System;
using System.Collections.Generic;
using System.Text;
using System.Collections.ObjectModel;

using thinktecture.ProxyModel.Remoting;
using thinktecture.ProxyModel.Web.Services;

namespace thinktecture.ProxyModel
{
  internal static class CommunicationPlatformFactory
  {
    #region fields

    //private static readonly List<BaseCommunicationPlatform> _communicationPlatforms = null;
    private static readonly CommunicationPlatformKeyedCollection _communicationPlatforms = null;

    #endregion
    #region ctors

    static CommunicationPlatformFactory()
    {
      // Load all communication platform configuration.
      //_communicationPlatforms = new List<BaseCommunicationPlatform>();
      //_communicationPlatforms.Add(new WellKnownClientRemotingCommunicationPlatform());
      //_communicationPlatforms.Add(new ActivatedClientRemotingCommunicationPlatform());
      _communicationPlatforms = new CommunicationPlatformKeyedCollection();
      _communicationPlatforms.Add(new RemotingCommunicationPlatform());
      _communicationPlatforms.Add(new WebServicesCommunicationPlatform());
    }

    #endregion
    #region methods

    private static bool Validate(Uri uri)
    {
      return Enum.IsDefined(typeof(UriScheme), uri.Scheme.ToLower());
    }
    internal static BaseCommunicationPlatform GetCommunicationPlatform(Type contractType)
    {
      return CommunicationPlatformFactory.GetCommunicationPlatform(contractType, null);
    }
    internal static BaseCommunicationPlatform GetCommunicationPlatform(Type contractType, Uri uri)
    {
      if ((uri != null) && (!CommunicationPlatformFactory.Validate(uri)))
      {
        throw new UriSchemeNotSupportedException(uri);
      }
      if (uri == null)
      {
        foreach (BaseCommunicationPlatform communicationPlatform in _communicationPlatforms)
        {
          if (communicationPlatform.IsDefined(contractType))
          {
            return communicationPlatform;
          }
        }
      }
      else
      {
        foreach (BaseCommunicationPlatform communicationPlatform in _communicationPlatforms)
        {
          if (communicationPlatform.IsDefined(contractType, uri))
          {
            return communicationPlatform;
          }
        }
      }
      return null;
    }
    internal static BaseCommunicationPlatform GetCommunicationPlatform(CommunicationTechnology communicationTechnology)
    {
      return _communicationPlatforms[communicationTechnology];
    }

    #endregion
  }
}
