#region (c) 2002-2006 by thinktecture
// **********************************************************************
//
//  2002-2006 by thinktecture, Ingo Rammer and Christian Weyer. All rights reserved.
//
// This copy of thinktecture is licensed to you under the terms described in the
// License.txt file included in this distribution.
//
// Author: 
// Christian Weyer (christian.weyer@thinktecture.com)
//
// **********************************************************************
#endregion
#region (c) 2006 by netfxfactory.org
// **********************************************************************
//
//  2006 by netfxfactory.org. All rights reserved.
//
// This copy is licensed to you under the terms described in the
// License.txt file included in this distribution.
//
// Author: 
// pascal.recchia@netfxfactory.org
// anthony.guerot@netfxfactory.org
// **********************************************************************
#endregion

using System;
using System.Text;
using System.Configuration;
using System.ComponentModel;
using System.Collections.Generic;

namespace thinktecture.ProxyModel.Configuration
{
  public sealed class TypeTypeConverter : TypeConverter
  {
    #region Methods

    public override object ConvertFrom(ITypeDescriptorContext context, System.Globalization.CultureInfo culture, object value)
    {
      return Type.GetType((string)value);
    }
    public override object ConvertTo(ITypeDescriptorContext context, System.Globalization.CultureInfo culture, object value, Type destinationType)
    {
      if (destinationType == typeof(string))
      {
        return ((Type)value).AssemblyQualifiedName;
      }
      return base.ConvertTo(context, culture, value, destinationType);
    }

    #endregion
  }

  public sealed class EndpointElement : ConfigurationElement
  {
    #region ctors

    public EndpointElement()
    {
    }

    #endregion
    #region props

    [ConfigurationProperty("name", DefaultValue = "", Options = ConfigurationPropertyOptions.IsKey)]
    public string Name
    {
      get { return (string)base["name"]; }
      set
      {
        if (string.IsNullOrEmpty(value))
        {
          value = string.Empty;
        }
        base["name"] = value;
      }
    }
    [ConfigurationProperty("address", Options = ConfigurationPropertyOptions.None)]
    public Uri Address
    {
      get { return (Uri)base["address"]; }
      set { base["address"] = value; }
    }
    [ConfigurationProperty("binding", Options = ConfigurationPropertyOptions.IsRequired)]
    public CommunicationTechnology Binding
    {
      get { return (CommunicationTechnology)base["binding"]; }
      set
      {
        base["binding"] = value;
      }
    }
    // TODO : Preserve the Contract property or not ?
    //[ConfigurationProperty("contract", Options = ConfigurationPropertyOptions.IsKey | ConfigurationPropertyOptions.IsRequired)]
    //public string Contract
    //{
    //  get { return (string)base["contract"]; }
    //  set
    //  {
    //    if (string.IsNullOrEmpty(value))
    //    {
    //      value = string.Empty;
    //    }
    //    base["contract"] = value;
    //  }
    //}
    [ConfigurationProperty("contract", IsRequired = true)]
    [TypeConverter(typeof(TypeTypeConverter))]
    public Type ContractType
    {
      get
      {
        Type __contractType = (Type)this["contract"];
        if (__contractType == null)
        {
          // HACK : Draft exception.
          throw new ConfigurationErrorsException(string.Format("The endpoint:{0} doesn't define a valid type name.", this.Name));
        }
        return __contractType;
      }
    }
    public string Key
    {
      get
      {
        return EndpointElement.GetKey(this.ContractType, this.Name);
      }
    }

    #endregion
    #region methods

    public static string GetKey(Type contractType)
    {
      return GetKey(contractType, null);
    }
    public static string GetKey(Type contractType, string endpointName)
    {
      return string.Format("contractType:{0};endpointName:{1}", contractType.AssemblyQualifiedName, endpointName);
    }

    #endregion
  }
}
