#region (c) 2002-2006 by thinktecture
// **********************************************************************
//
//  2002-2006 by thinktecture, Ingo Rammer and Christian Weyer. All rights reserved.
//
// This copy of thinktecture is licensed to you under the terms described in the
// License.txt file included in this distribution.
//
// Author: 
// Christian Weyer (christian.weyer@thinktecture.com)
//
// **********************************************************************
#endregion
#region (c) 2006 by netfxfactory.org
// **********************************************************************
//
//  2006 by netfxfactory.org. All rights reserved.
//
// This copy is licensed to you under the terms described in the
// License.txt file included in this distribution.
//
// Author: 
// pascal.recchia@netfxfactory.org
// anthony.guerot@netfxfactory.org
// **********************************************************************
#endregion

using System;
using System.Collections.Generic;
using System.Text;

namespace thinktecture.ProxyModel
{
  public abstract class DependencyObject
  {
    #region fields

    private static readonly DependencyProperty ParentDependencyObjectProperty;
    private Dictionary<DependencyProperty, object> _dependencyPropertyValues;

    #endregion
    #region ctors

    static DependencyObject()
    {
      DependencyObject.ParentDependencyObjectProperty = DependencyProperty.Register("ParentDependencyObject", typeof(DependencyObject), typeof(DependencyObject));
    }
    protected DependencyObject()
    {
      _dependencyPropertyValues = new Dictionary<DependencyProperty, object>();
    }

    #endregion
    #region props

    internal IDictionary<DependencyProperty, object> DependencyPropertyValues
    {
      get { return _dependencyPropertyValues; }
    }
    protected DependencyObject ParentDependencyObject
    {
      get
      {
        return (DependencyObject)this.GetValue(DependencyObject.ParentDependencyObjectProperty);
      }
    }
    
    #endregion
    #region methods

    private DependencyObject GetOwner(DependencyObject dependencyObject, Type ownerType)
    {
      DependencyObject __dependencyObject = dependencyObject;
      while (__dependencyObject != null)
      {
        if (ownerType.IsInstanceOfType(__dependencyObject))
        {
          return __dependencyObject;
        }
        __dependencyObject = __dependencyObject.ParentDependencyObject;
      }
      return null;
    }
    // TODO : virtual see soaphttpclientprotocol.Proxy ... can not be a dependencyproperty. see setboundvalue .. on oblige  appeler un custom bound apres nos validations.
    protected void SetValue(DependencyProperty dependencyProperty, object value)
    {
      if (!dependencyProperty.PropertyType.IsInstanceOfType(value))
      {
        throw new DependencyPropertyTypeMismatchException(dependencyProperty, value);
      }
      if (!dependencyProperty.OwnerType.IsInstanceOfType(this))
      {
        DependencyObject __owner = this.GetOwner(this, dependencyProperty.OwnerType);
        if (__owner != null)
        {
          __owner.SetValue(dependencyProperty, value);
        }
        return;
      }
      object __object = null;
      if (!this.DependencyPropertyValues.TryGetValue(dependencyProperty, out __object))
      {
        this.DependencyPropertyValues.Add(dependencyProperty, value);
      }
      else
      {
        this.DependencyPropertyValues[dependencyProperty] = value;
      }
      this.SetBoundValue(dependencyProperty, value);
    }
    protected object GetValue(DependencyProperty dependencyProperty)
    {
      if (!dependencyProperty.OwnerType.IsInstanceOfType(this))
      {
        DependencyObject __owner = this.GetOwner(this, dependencyProperty.OwnerType);
        if (__owner == null)
        {
          return null;
        }
        return __owner.GetValue(dependencyProperty);
      }
      object __object = this.GetBoundValue(dependencyProperty);
      if (__object != null)
      {
        return __object;
      }
      if (!this.DependencyPropertyValues.TryGetValue(dependencyProperty, out __object))
      {
        if (dependencyProperty.DefaultValue == null)
        {
          return ((dependencyProperty.PropertyType.IsValueType) ? Activator.CreateInstance(dependencyProperty.PropertyType) : null);
        }
        return dependencyProperty.DefaultValue;
      }
      return __object;
    }
    internal void Attach(DependencyObject dependencyObject)
    {
      if ((this == dependencyObject) || ((this.ParentDependencyObject != null) && (this.ParentDependencyObject == dependencyObject)))
      {
        throw new CyclicReferenceException();
      }
      dependencyObject.SetValue(DependencyObject.ParentDependencyObjectProperty, this);
    }

    protected virtual object GetBoundValue(DependencyProperty dependencyProperty) { return null; } //wpf::geteffectivevalue
    protected virtual void SetBoundValue(DependencyProperty dependencyProperty, object value) { } //wpf::UpdateEffectiveValue

    #endregion
  }
}
