#region (c) 2002-2006 by thinktecture
// **********************************************************************
//
//  2002-2006 by thinktecture, Ingo Rammer and Christian Weyer. All rights reserved.
//
// This copy of thinktecture is licensed to you under the terms described in the
// License.txt file included in this distribution.
//
// Author: 
// Christian Weyer (christian.weyer@thinktecture.com)
//
// **********************************************************************
#endregion
#region (c) 2006 by netfxfactory.org
// **********************************************************************
//
//  2006 by netfxfactory.org. All rights reserved.
//
// This copy is licensed to you under the terms described in the
// License.txt file included in this distribution.
//
// Author: 
// pascal.recchia@netfxfactory.org
// anthony.guerot@netfxfactory.org
// **********************************************************************
#endregion

using System;
using System.Text;
using System.Runtime.Remoting;
using System.Collections.Generic;
using System.Collections.ObjectModel;

using thinktecture.ProxyModel.Configuration;

namespace thinktecture.ProxyModel
{
  // TODO : EndpointElement property
  // TODO : Select the communication platform in parsing the custom attributes of the specified contract type : [WebService]
  // TODO : A tool which generate interface from a wsdl document.
  public sealed class ProxyFactory<TContract> : DependencyObject
  {
    #region fields

    private Uri _uri = null;
    private BaseCommunicationPlatform _communicationPlatform;

    #endregion
    #region ctors

    /// <summary>
    /// Initializes a new instance of the <see cref="ProxyModel.ProxyFactory{TContract}"/> class.
    /// </summary>
    public ProxyFactory()
    {
      // Get the communication platform associated with the specified service contract.
      // Test if an endpoint is associated with this contract.
      EndpointElement __endpointElement = ConfigurationSettings.GetEndpointElement(typeof(TContract));
      if (__endpointElement != null)
      {
        _communicationPlatform = CommunicationPlatformFactory.GetCommunicationPlatform(__endpointElement.Binding);
        _uri = __endpointElement.Address;
      }
      else
      {
        // No endpoint was found.
        // Test if a communication platform is associated with this contract. //remoting
        _communicationPlatform = CommunicationPlatformFactory.GetCommunicationPlatform(typeof(TContract));
      }
      if (_communicationPlatform == null)
      {
        throw new CommunicationPlatformNotFoundException(typeof(TContract));
      }

      // Attach the proxy factory to the communication platform to take advantage of the DependencyObject mechanism.
      _communicationPlatform.Attach(this);

    }
    // /// <exception cref="UriSchemeNotSupportedException">The schema name of the uri associated with the service declared by the contract TContract is not supported.</exception>
    public ProxyFactory(Uri uri)
    {
      _uri = uri;
      // Get the communication platform associated with the specified uri and the specified service contract.
      _communicationPlatform = CommunicationPlatformFactory.GetCommunicationPlatform(typeof(TContract), _uri);
      if (_communicationPlatform == null)
      {
        throw new CommunicationPlatformNotFoundException(typeof(TContract), _uri);
      }

      // Attach the proxy factory to the communication platform to take advantage of the DependencyObject mechanism.
      _communicationPlatform.Attach(this);
    }
    public ProxyFactory(string endpointName)
    {
      EndpointElement __endpointElement = ConfigurationSettings.GetEndpointElement(typeof(TContract), endpointName);
      if (__endpointElement == null)
      {
        throw new EndpointNotFoundException(endpointName);
      }
      _communicationPlatform = CommunicationPlatformFactory.GetCommunicationPlatform(__endpointElement.Binding);
      _uri = __endpointElement.Address;

      if (_communicationPlatform == null)
      {
        throw new CommunicationPlatformNotFoundException(typeof(TContract));
      }

      // Attach the proxy factory to the communication platform to take advantage of the DependencyObject mechanism.
      _communicationPlatform.Attach(this);
    }
    public ProxyFactory(CommunicationTechnology binding, Uri uri)
    {
      _communicationPlatform = CommunicationPlatformFactory.GetCommunicationPlatform(binding);
      _uri = uri;
      if (_communicationPlatform == null)
      {
        throw new CommunicationPlatformNotFoundException(typeof(TContract), _uri);
      }

      // Attach the proxy factory to the communication platform to take advantage of the DependencyObject mechanism.
      _communicationPlatform.Attach(this);
    }

    #endregion
    #region props

    public CommunicationTechnology Binding
    {
      get
      {
        return _communicationPlatform.CommunicationTechnology;
      }
    }
    public Uri uri
    {
      get
      {
        return _uri;
      }
    }

    #endregion
    #region methods

    public TContract CreateProxy()
    {
      TContract __proxy = (TContract)(_uri == null ? _communicationPlatform.GetProxy(typeof(TContract)) : _communicationPlatform.GetProxy(typeof(TContract), _uri));
      if (__proxy == null)
      {
        throw new ProxyInvocationException(typeof(TContract), this.Binding);
      }
      return __proxy;
    }
    public void SetPropertyValue(DependencyProperty dependencyProperty, object value)
    {
      this.SetValue(dependencyProperty, value);
    }
    public object GetPropertyValue(DependencyProperty dependencyProperty)
    {
      return this.GetValue(dependencyProperty);
    }
    public T GetPropertyValue<T>(DependencyProperty dependencyProperty)
    {
      if (dependencyProperty.PropertyType != typeof(T))
      {
        // TODO : exception messagge.
        throw new InvalidCastException();
      }
      return (T)this.GetValue(dependencyProperty);
    }

    #endregion
  }
}
