#region (c) 2002-2006 by thinktecture
// **********************************************************************
//
//  2002-2006 by thinktecture, Ingo Rammer and Christian Weyer. All rights reserved.
//
// This copy of thinktecture is licensed to you under the terms described in the
// License.txt file included in this distribution.
//
// Author: 
// Christian Weyer (christian.weyer@thinktecture.com)
//
// **********************************************************************
#endregion
#region (c) 2006 by netfxfactory.org
// **********************************************************************
//
//  2006 by netfxfactory.org. All rights reserved.
//
// This copy is licensed to you under the terms described in the
// License.txt file included in this distribution.
//
// Author: 
// pascal.recchia@netfxfactory.org
// anthony.guerot@netfxfactory.org
// **********************************************************************
#endregion

using System;
using System.IO;
using System.Xml;
using System.Text;
using System.Xml.XPath;
using System.Runtime.Remoting;
using System.Collections.Generic;

namespace thinktecture.ProxyModel.Remoting
{
  internal sealed class RemotingCommunicationPlatform : BaseCommunicationPlatform
  {
    #region fields

    private const string ATTR_CHANNEL_SECURE = "configuration/system.runtime.remoting//channels/channel/@secure";
    private static readonly List<ActivatedClientTypeEntry> _activatedClientTypeEntries = null;
    private static readonly XPathExpression _getChannelSecureAttribute = XPathExpression.Compile(ATTR_CHANNEL_SECURE);

    #endregion
    #region ctors

    static RemotingCommunicationPlatform()
    {
      try
      {
        Configure();
      }
      catch (RemotingException)
      {
        // Do nothing. The RemotingConfiguration could have been configure previously.
      }

      // Load all remotely actived client type entries.
      _activatedClientTypeEntries = new List<ActivatedClientTypeEntry>(RemotingConfiguration.GetRegisteredActivatedClientTypes());
    }

    #endregion
    #region props

    public override CommunicationTechnology CommunicationTechnology
    {
      get { return CommunicationTechnology.Remoting; }
    }

    #endregion
    #region methods

    private static void Configure()
    {
      System.Configuration.Configuration __configuration = thinktecture.ProxyModel.Configuration.ConfigurationSettings.GetConfiguration();
      string __configurationFilePath = __configuration.FilePath;

      // If the configuration file doesn't exist, stop at this point.
      if (!File.Exists(__configurationFilePath))
      {
        return;
      }

      // Parse the configuration file to determine if it must to set to true the ensureSecurity parameter of the RemotingConfiguration.Configure method.
      bool __ensureSecurity = false;
      using (XmlReader __xmlReader = new XmlTextReader(__configurationFilePath))
      {
        XPathDocument __bodyDocument = new XPathDocument(__xmlReader);
        XPathNavigator __navigator = __bodyDocument.CreateNavigator();
        XPathNodeIterator __xpathNodeIterator = __navigator.Select(_getChannelSecureAttribute);
        if (__xpathNodeIterator.Count != 0)
        {
          __xpathNodeIterator.MoveNext();
          __ensureSecurity = (bool)__xpathNodeIterator.Current.ValueAs(typeof(Boolean));
        }
      }
      RemotingConfiguration.Configure(__configurationFilePath, __ensureSecurity);
    }
    private bool TryGetValue(Type contractType, Uri uri, out WellKnownClientTypeEntry wellKnownClientTypeEntry)
    {
      wellKnownClientTypeEntry = RemotingConfiguration.IsWellKnownClientType(contractType);
      if ((wellKnownClientTypeEntry != null) && ((uri == null) || (uri != null) && (wellKnownClientTypeEntry.ObjectUrl == uri.AbsoluteUri)))
      {
        return true;
      }
      wellKnownClientTypeEntry = null;
      return false;
    }
    private bool TryGetValue(Type contractType, Uri uri, out ActivatedClientTypeEntry activatedClientTypeEntry)
    {
      // TODO : Check if the item has a well type ... else return error ? verify remoting configuration ?
      // TODO : Throw error if the ObjectType is null or is not interface ?
      activatedClientTypeEntry = _activatedClientTypeEntries.Find(delegate(ActivatedClientTypeEntry item) 
        {
          Uri __applicationUri = new Uri(item.ApplicationUrl);
          return ((item.ObjectType.GetInterface(contractType.FullName, false) != null) && ((uri == null) || (uri != null) && (__applicationUri.IsBaseOf(uri))));
        });
      return (activatedClientTypeEntry != null);
    }
    private bool TryGetValue(Type contractType, Uri uri, out TypeEntry typeEntry)
    {
      WellKnownClientTypeEntry __wellKnownClientTypeEntry = null;
      if (TryGetValue(contractType, uri, out __wellKnownClientTypeEntry))
      {
        typeEntry = __wellKnownClientTypeEntry;
        return true;
      }
      ActivatedClientTypeEntry __activatedClientTypeEntry = null;
      if (TryGetValue(contractType, uri, out __activatedClientTypeEntry))
      {
        typeEntry = __activatedClientTypeEntry;
        return true;
      }
      typeEntry = null;
      return false;
    }
    public override bool IsDefined(Type contractType, Uri uri)
    {
      TypeEntry __typeEntry = null;
      return TryGetValue(contractType, uri, out __typeEntry);
    }
    public override object GetProxy(Type contractType, Uri uri)
    {
      WellKnownClientTypeEntry __wellKnownClientTypeEntry = null;
      if (TryGetValue(contractType, uri, out __wellKnownClientTypeEntry))
      {
        return Activator.GetObject(__wellKnownClientTypeEntry.ObjectType, __wellKnownClientTypeEntry.ObjectUrl);
      }
      ActivatedClientTypeEntry __activatedClientTypeEntry = null;
      if (TryGetValue(contractType, uri, out __activatedClientTypeEntry))
      {
        return Activator.CreateInstance(__activatedClientTypeEntry.ObjectType);
      }
      return null;
    }

    #endregion
  }
}
