#region (c) 2002-2006 by thinktecture
// **********************************************************************
//
//  2002-2006 by thinktecture, Ingo Rammer and Christian Weyer. All rights reserved.
//
// This copy of thinktecture is licensed to you under the terms described in the
// License.txt file included in this distribution.
//
// Author: 
// Christian Weyer (christian.weyer@thinktecture.com)
//
// **********************************************************************
#endregion
#region (c) 2006 by NetFxFactory.org
// **********************************************************************
//
//  2006 by NetFxFactory.org. All rights reserved.
//
// This copy is licensed to you under the terms described in the
// License.txt file included in this distribution.
//
// Author: 
// pascal.recchia@netfxfactory.org
// anthony.guerot@netfxfactory.org
// **********************************************************************
#endregion

using System;
using System.Globalization;
using System.IO;
using System.Reflection;
using System.Security.Cryptography;
using System.Text;
using System.Configuration;

namespace thinktecture.ProxyModel.Web.Services
{
  /// <summary>
  /// Provides methods for managing cached proxy assemblies.
  /// </summary>
  internal static class CompiledAssemblyCacheHelper
  {
    #region consts

    private const string TEMPDLLEXTENSION = "_ProxyFactory_tmp.dll";

    #endregion
    #region methods

    /// <summary>
    /// Gets the MD5 sum.
    /// </summary>
    /// <param name="stringToHash">String to hash.</param>
    /// <returns>The MD5 sum.</returns>
    private static string GetMd5Sum(string stringToHash)
    {
      // First we need to convert the string into bytes, which
      // means using a text encoder
      Encoder enc = Encoding.Unicode.GetEncoder();

      // Create a buffer large enough to hold the string
      byte[] unicodeText = new byte[stringToHash.Length * 2];
      enc.GetBytes(stringToHash.ToCharArray(), 0, stringToHash.Length, unicodeText, 0, true);

      // Now that we have a byte array we can ask the CSP to hash it
      MD5 md5 = new MD5CryptoServiceProvider();
      byte[] result = md5.ComputeHash(unicodeText);

      // Build the final string by converting each byte
      // into hex and appending it to a StringBuilder
      StringBuilder sb = new StringBuilder();
      for (int i = 0; i < result.Length; i++)
      {
        sb.Append(result[i].ToString("X2", CultureInfo.CurrentCulture));
      }

      return sb.ToString();
    }
    /// <summary>
    /// Checks if it exists a proxy assembly in the cache for the specified wsdl document.
    /// </summary>
    /// <returns>The assembly if it exists, otherwise null.</returns>
    internal static Assembly CheckCacheForAssembly(string wsdl)
    {
      StringBuilder __stringBuilder = new StringBuilder(Path.GetTempPath());
      __stringBuilder.Append(GetMd5Sum(wsdl));
      __stringBuilder.Append(TEMPDLLEXTENSION);
      string __path = __stringBuilder.ToString();
      if (File.Exists(__path))
      {
        return Assembly.LoadFrom(__path);
      }
      return null;
    }
    /// <summary>
    /// Clears the cache for the specified WSDL document.
    /// </summary>
    /// <param name="wsdlLocation">The WSDL document location.</param>
    internal static void ClearCache(string wsdlLocation)
    {
      // clear the cached assembly file for this WSDL
      try
      {
        StringBuilder __stringBuilder = new StringBuilder(Path.GetTempPath());
        __stringBuilder.Append(GetMd5Sum(wsdlLocation));
        __stringBuilder.Append(TEMPDLLEXTENSION);
        string __path = __stringBuilder.ToString();
        File.Delete(__path);
      }
      catch (Exception ex)
      {
        // TODO : Localize this exception message.
        throw new TemporaryCacheException("Problem occured when trying to clear temporary local assembly cache for WSDL: " + wsdlLocation + ".", ex);
      }
    }
    /// <summary>
    /// Clears all cached proxy assemblies.
    /// </summary>
    internal static void ClearAllCached()
    {
      DirectoryInfo __temporaryDirectoryInfo = new DirectoryInfo(Path.GetTempPath());
      FileInfo[] __dllFiles = __temporaryDirectoryInfo.GetFiles(string.Concat("*",TEMPDLLEXTENSION));
      foreach (FileInfo fileInfo in __dllFiles)
      {
        try
        {
          fileInfo.Delete();
        }
        catch (Exception ex)
        {
          // TODO : Localize this exception message.
          throw new TemporaryCacheException("Problem occurred when trying to clear temporary local assembly cache.", ex);
        }
      }
    }
    /// <summary>
    /// Renames the temporary proxy assembly to .
    /// </summary>
    /// <param name="pathToAssembly">Path to assembly.</param>
    internal static void RenameTempAssembly(string pathToAssembly, string wsdl)
    {
      string __directoryName = Path.GetDirectoryName(pathToAssembly);
      StringBuilder __stringBuilder = new StringBuilder(__directoryName);
      __stringBuilder.Append(@"\");
      __stringBuilder.Append(GetMd5Sum(wsdl));
      __stringBuilder.Append(TEMPDLLEXTENSION);
      File.Copy(pathToAssembly, __stringBuilder.ToString(), true);
    }

    #endregion
  }
}
