#region (c) 2002-2006 by thinktecture
// **********************************************************************
//
//  2002-2006 by thinktecture, Ingo Rammer and Christian Weyer. All rights reserved.
//
// This copy of thinktecture is licensed to you under the terms described in the
// License.txt file included in this distribution.
//
// Author: 
// Christian Weyer (christian.weyer@thinktecture.com)
//
// **********************************************************************
#endregion
#region (c) 2006 by netfxfactory.org
// **********************************************************************
//
//  2006 by netfxfactory.org. All rights reserved.
//
// This copy is licensed to you under the terms described in the
// License.txt file included in this distribution.
//
// Author: 
// pascal.recchia@netfxfactory.org
// anthony.guerot@netfxfactory.org
// **********************************************************************
#endregion

using System;
using System.Collections.Generic;
using System.Text;
using System.Net;

namespace thinktecture.ProxyModel.Web.Services
{
  /// <summary>
  /// Defines the contract that ASP.NET XML Web service client proxies implements.
  /// </summary>
  /// <seealso cref="System.Web.Services.Protocols.WebClientProtocol"/>
  public interface IWebClientProtocol
  {
    #region props

    /// <summary>
    /// Gets or sets the name of the connection group for the request.
    /// </summary>
    string ConnectionGroupName { get; set; }
    /// <summary>
    /// Gets or sets security credentials for XML Web service client authentication.
    /// </summary>
    ICredentials Credentials { get; set; }
    /// <summary>
    /// Gets or sets whether pre-authentication is enabled.
    /// </summary>
    bool PreAuthenticate { get; set; }
    /// <summary>
    /// The <see cref="Encoding"/> used to make the client request to the XML Web service.
    /// </summary>
    Encoding RequestEncoding { get; set; }
    /// <summary>
    /// Indicates the time an XML Web service client waits for a synchronous XML Web service request to complete (in milliseconds).
    /// </summary>
    int Timeout { get; set; }
    /// <summary>
    /// Gets or sets the base URL of the XML Web service the client is requesting.
    /// </summary>
    string Url { get; set; }
    /// <summary>
    /// Gets or sets a value that indicates whether to set the Credentials property to the value of the <see cref="System.Net.CredentialCache.DefaultCredentials"/> property.
    /// </summary>
    bool UseDefaultCredentials { get; set; }

    #endregion
    #region methods

    /// <summary>
    /// Cancels a request to an XML Web service method. 
    /// </summary>
    void Abort();

    #endregion
  }
}
