#region (c) 2002-2006 by thinktecture
// **********************************************************************
//
//  2002-2006 by thinktecture, Ingo Rammer and Christian Weyer. All rights reserved.
//
// This copy of thinktecture is licensed to you under the terms described in the
// License.txt file included in this distribution.
//
// Author: 
// Christian Weyer (christian.weyer@thinktecture.com)
//
// **********************************************************************
#endregion
#region (c) 2006 by netfxfactory.org
// **********************************************************************
//
//  2006 by netfxfactory.org. All rights reserved.
//
// This copy is licensed to you under the terms described in the
// License.txt file included in this distribution.
//
// Author: 
// pascal.recchia@netfxfactory.org
// anthony.guerot@netfxfactory.org
// **********************************************************************
#endregion

using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Web.Services3.Addressing;
using Microsoft.Web.Services3;
using Microsoft.Web.Services3.Design;
using Microsoft.Web.Services3.Security.Tokens;

namespace thinktecture.ProxyModel.Web.Services
{
  /// <summary>
  /// Defines the contract that ASP.NET XML Web service client proxies, the Web Services Enhancements (WSE), implements.
  /// </summary>
  /// <seealso cref="WebServicesClientProtocol"/>
  public interface IWebServicesClientProtocol : ISoapHttpClientProtocol
  {
    #region props

    /// <summary>
    /// Gets or sets the endpoint of the SOAP message recipient.
    /// </summary>
    EndpointReference Destination { get; set; }
    /// <summary>
    /// Gets the object that allows custom filtering of messages.
    /// </summary>
    Pipeline Pipeline { get; set; }
    /// <summary>
    /// Gets security, routing, and other XML Web services architecture-specific information that is associated with a SOAP request.
    /// </summary>
    SoapContext RequestSoapContext { get; }
    /// <summary>
    /// Gets or sets a value indicating whether SOAP requests are MTOM-encoded.
    /// </summary>
    bool RequireMtom { get; set; }
    /// <summary>
    /// Gets security, routing, and other XML Web services architecture-specific information that is associated with a SOAP response.
    /// </summary>
    SoapContext ResponseSoapContext { get; }

    #endregion
    #region methods

    /// <summary>
    /// Specifies the policy for a SOAP message exchange using a policy that is specified in application code.
    /// </summary>
    /// <param name="policy">A <see cref="Policy"/> that specifies the policy for a SOAP message exchange.</param>
    void SetPolicy(Policy policy);
    /// <summary>
    /// Specifies the policy for a SOAP message exchange using a policy that is specified in a policy file.
    /// </summary>
    /// <param name="policyName">The name of a policy that is defined in a policy file for a SOAP message exchange.</param>
    void SetPolicy(string policyName);
    /// <summary>
    /// Specifies the security credentials of the Web service client.
    /// </summary>
    /// <typeparam name="TSecurityToken">The type of a SecurityToken that represents the security credentials of the Web service client.</typeparam>
    /// <param name="clientToken">A SecurityToken that represents the security credentials of the Web service client.</param>
    void SetClientCredential<TSecurityToken>(TSecurityToken clientToken) where TSecurityToken : SecurityToken;
    /// <summary>
    /// Gets the security credentials of the Web service client.
    /// </summary>
    /// <typeparam name="TSecurityToken">The type of a A SecurityToken that represents the security credentials of the Web service client.</typeparam>
    /// <returns>A SecurityToken that represents the security credentials of the Web service client.</returns>
    TSecurityToken GetClientCredential<TSecurityToken>() where TSecurityToken : SecurityToken;

    #endregion
  }
}
