#region (c) 2002-2006 by thinktecture
// **********************************************************************
//
//  2002-2006 by thinktecture, Ingo Rammer and Christian Weyer. All rights reserved.
//
// This copy of thinktecture is licensed to you under the terms described in the
// License.txt file included in this distribution.
//
// Author: 
// Christian Weyer (christian.weyer@thinktecture.com)
//
// **********************************************************************
#endregion
#region (c) 2006 by netfxfactory.org
// **********************************************************************
//
//  2006 by netfxfactory.org. All rights reserved.
//
// This copy is licensed to you under the terms described in the
// License.txt file included in this distribution.
//
// Author: 
// pascal.recchia@netfxfactory.org
// anthony.guerot@netfxfactory.org
// **********************************************************************
#endregion

using System;
using System.Text;
using System.Reflection;
using System.Collections.Generic;

namespace thinktecture.ProxyModel.Web.Services
{
  internal sealed class TypeExporter
  {
    #region fields

    private List<Type> _types = null;
    private Type _interfaceType = null;

    #endregion
    #region ctors

    internal TypeExporter(Type interfaceType)
    {
      if (!interfaceType.IsInterface)
      {
        throw new ArgumentException("interfaceType must be an intetface", "interfaceType");
      }
      _interfaceType = interfaceType;
      _types = new List<Type>();
    }

    #endregion
    #region methods

    private void Handle(Type type)
    {
      // Don't handle primitive type or parameter type.
      if ((type.IsPrimitive) || (type.IsGenericParameter))
      {
        return;
      }
      // If the type is an array, handle the type returned by the call of the GetElementType method.
      if (type.IsArray)
      {
        type = type.GetElementType();
      }
      if (!_types.Contains(type))
      {
        _types.Add(type);
        this.Export(type);
      }
    }
    private MemberInfo[] GetPublicPropertyInfosAndPublicFieldInfos(Type type)
    {
      List<MemberInfo> __memberInfos = new List<MemberInfo>(type.GetProperties(BindingFlags.Public | BindingFlags.Instance));
      __memberInfos.AddRange(type.GetFields(BindingFlags.Public | BindingFlags.Instance));
      return __memberInfos.ToArray();
    }
    private void Export(Type type)
    {
      //TODO : Test if it must parse the BaseType property, too.
      MemberInfo[] __memberInfos = this.GetPublicPropertyInfosAndPublicFieldInfos(type);
      foreach (MemberInfo memberInfo in __memberInfos)
      {
        Type __memberInfoType = null;
        switch (memberInfo.MemberType)
        {
          case MemberTypes.Field:
            __memberInfoType = ((FieldInfo)memberInfo).FieldType;
            break;
          case MemberTypes.Property:
            __memberInfoType = ((PropertyInfo)memberInfo).PropertyType;
            break;
        }
        if (__memberInfoType != null)
        {
          this.Handle(__memberInfoType);
        }
      }
    }
    private void ExportInterface(Type interfaceType)
    {
      //HACK: the type is necessary an interface.
      //parses the methods declared by the interface.
      MethodInfo[] __methodInfos = interfaceType.GetMethods();
      foreach (MethodInfo methodInfo in __methodInfos)
      {
        this.Handle(methodInfo.ReturnType);
        foreach (ParameterInfo parameterInfo in methodInfo.GetParameters())
        {
          this.Handle(parameterInfo.ParameterType);
        }
      }
      //do the same job for the interfaces associated with the specifid interfaceType.
      Type[] __baseInterfaceTypes = interfaceType.GetInterfaces();
      foreach (Type baseInterfaceType in __baseInterfaceTypes)
      {
        this.ExportInterface(baseInterfaceType);
      }
    }
    internal Type[] Export()
    {
      this.ExportInterface(_interfaceType);
      return _types.ToArray();
    }

    #endregion
  }
}
