#region (c) 2002-2006 by thinktecture
// **********************************************************************
//
//  2002-2006 by thinktecture, Ingo Rammer and Christian Weyer. All rights reserved.
//
// This copy of thinktecture is licensed to you under the terms described in the
// License.txt file included in this distribution.
//
// Author: 
// Christian Weyer (christian.weyer@thinktecture.com)
//
// **********************************************************************
#endregion
#region (c) 2006 by netfxfactory.org
// **********************************************************************
//
//  2006 by netfxfactory.org. All rights reserved.
//
// This copy is licensed to you under the terms described in the
// License.txt file included in this distribution.
//
// Author: 
// pascal.recchia@netfxfactory.org
// anthony.guerot@netfxfactory.org
// **********************************************************************
#endregion

using System;
using System.CodeDom;
using System.CodeDom.Compiler;
using System.Collections;
using System.Globalization;
using System.IO;
using System.Net;
using System.Reflection;
using System.Text;
using System.Web;
using System.Web.Services.Description;
using System.Web.Services.Discovery;
using System.Xml;
using System.Xml.Schema;
using System.Xml.Serialization;
using Microsoft.CSharp;
using System.Data;
using System.Collections.Generic;
using System.Net.Cache;
using System.Web.Services.Protocols;
using System.Web.Services;

namespace thinktecture.ProxyModel.Web.Services
{
  // TODO : Test in web environment.
  internal sealed class WebServiceProxyBuilder : DependencyObject
  {
    #region fields

    private Type _contractType;
    private Assembly _generatedAssembly;
    private Uri _wsdlUri;
    private string _wsdlTemporaryFilePath;
    private ICredentials _credentials;
    private IWebProxy _proxy;

    #endregion
    #region ctors

    /// <summary>
    /// Initializes a new instance of the <see cref="WebServiceProxyBuilder"/> class with a specified WSDL document location and its associated contract type. 
    /// </summary>
    /// <param name="wsdlLocation">The WSDL document location.</param>
    /// <param name="contractType">The contract type.</param>
    public WebServiceProxyBuilder(string wsdlLocation, Type contractType)
    {
      _wsdlUri = new Uri(wsdlLocation);
      if (_wsdlUri.Segments.Length < 2)
      {
        throw new UriFormatException("The WSDL document location: " + wsdlLocation + " is not valid. It must have at least two uri segmentst.");
      }
      _contractType = contractType;
    }

    #endregion
    #region props

    // HACK : Develop a specific WSDL Downloader class ?
    /// <summary>
    /// Get the temporary file path where is downloaded the WSDL document.
    /// </summary>
    private string WsdlTemporaryFilePath
    {
      get
      {
        if (string.IsNullOrEmpty(_wsdlTemporaryFilePath))
        {
          string __fileName = _wsdlUri.Segments[_wsdlUri.Segments.Length - 1];
          _wsdlTemporaryFilePath = string.Concat(Path.GetTempPath(), __fileName);
        }
        return _wsdlTemporaryFilePath;
      }
    }
    internal ICredentials Credentials
    {
      get { return _credentials; }
      set { _credentials = value; }
    }
    internal IWebProxy Proxy
    {
      get { return _proxy; }
      set { _proxy = value; }
    }

    // TODO : Set the protocol from a custom attribute declared from the contract type.
    ///// <summary>
    ///// Gets or sets the name of the protocol.
    ///// </summary>
    ///// <value></value>
    //public Protocol ProtocolName
    //{
    //  get
    //  {
    //    switch (protocolName)
    //    {
    //      case "HttpGet":
    //        return Protocol.HttpGet;
    //      case "HttpPost":
    //        return Protocol.HttpPost;
    //      case "Soap":
    //        return Protocol.HttpSoap;
    //      default:
    //        return Protocol.HttpSoap;
    //    }
    //  }
    //  set
    //  {
    //    switch (value)
    //    {
    //      case Protocol.HttpGet:
    //        protocolName = "HttpGet";
    //        break;
    //      case Protocol.HttpPost:
    //        protocolName = "HttpPost";
    //        break;
    //      case Protocol.HttpSoap:
    //        protocolName = "Soap";
    //        break;
    //    }
    //  }
    //}

    #endregion
    #region methods

    /// <summary>
    /// Get the base type of the generated proxy class according to the specified web service technology (Asmx, Wse...)
    /// </summary>
    /// <param name="webServiceTechnologies">The used web service technology</param>
    /// <returns>The base type to use.</returns>
    /// <remarks>Operation 4</remarks>
    private Type GetBaseType(WebServiceTechnologies webServiceTechnologies)
    {
      switch (webServiceTechnologies)
      {
        case WebServiceTechnologies.Asmx:
          return typeof(SoapHttpClientProtocol);
        case WebServiceTechnologies.Wse:
          return Wse.GetWebServicesClientProtocolType();
      }
      // TODO : Localize this exception message.
      throw new InvalidOperationException("The specified web service technology is not yet supported.");
    }
    /// <summary>
    /// Reference assemblies associated to the specified contract interface type in order to compile the generated assembly.
    /// </summary>
    /// <param name="compilerParameters">The parameters used to invoke the compiler.</param>
    /// <param name="assembly">The assembly which references assemblies.</param>
    /// <remarks>Operation 5</remarks>
    private void ReferenceAssemblies(CompilerParameters compilerParameters, Assembly assembly)
    {
      if (compilerParameters.ReferencedAssemblies.Contains(assembly.Location))
      {
        return;
      }
      string __fileName = Path.GetFileName(assembly.Location);
      if (compilerParameters.ReferencedAssemblies.Contains(__fileName))
      {
        return;
      }
      compilerParameters.ReferencedAssemblies.Add(assembly.Location);
      foreach (AssemblyName __assemblyName in assembly.GetReferencedAssemblies())
      {
        this.ReferenceAssemblies(compilerParameters, Assembly.Load(__assemblyName.FullName));
      }
    }
    /// <summary>
    /// Builds the assembly from the WSDL document associated with the <see cref="WebServiceProxyBuilder"/> instance.
    /// </summary>
    /// <returns>The generated proxy assembly.</returns>
    /// <remarks>Operation 3</remarks>
    private Assembly Generate(WebServiceTechnologies webServiceTechnologies, string protocolName)
    {
      // Discover the wsdl document and its references. 
      DiscoveryClientProtocol __discoveryClientProtocol = new DiscoveryClientProtocol();
      __discoveryClientProtocol.DiscoverAny(this.WsdlTemporaryFilePath);
      //__discoveryClientProtocol.ResolveAll(); // HACK: It's not currently necessary to valid the discovery documents.

      // Create a namespace and a unit for proxy client compilation.
      CodeNamespace __namespace = new CodeNamespace();
      CodeCompileUnit __compileUnit = new CodeCompileUnit();
      __compileUnit.Namespaces.Add(__namespace);

      // Create a web reference using the WSDL collection. The discovery documents are associated with the previously created code namespace.
      WebReference __webReference = new WebReference(__discoveryClientProtocol.Documents, __namespace);
      __webReference.ProtocolName = protocolName;

      // Create a web reference collection which includes the previously instancied WebReference object.
      WebReferenceCollection __webReferences = new WebReferenceCollection();
      __webReferences.Add(__webReference);

      // Define options for the compilation of the proxy client. 
      CodeDomProvider __csharpCodeProvider = CodeDomProvider.CreateProvider("CSharp");
      WebReferenceOptions __webReferenceOptions = new WebReferenceOptions();
      __webReferenceOptions.Style = ServiceDescriptionImportStyle.Client;
      __webReferenceOptions.CodeGenerationOptions = CodeGenerationOptions.GenerateProperties;

      // Declares our custom SchemaImporterExtension (It map the wsdl document object type with the types associated with the supplied contract interface type.
      Type __schemaImporterExtensionGeneric = typeof(SchemaImporterExtension<>);
      Type __schemaImporterExtension = __schemaImporterExtensionGeneric.MakeGenericType(_contractType);
      __webReferenceOptions.SchemaImporterExtensions.Add(__schemaImporterExtension.AssemblyQualifiedName);

      // Build the CodeDOM tree for the proxy client associated with the specified wsdl document.
      ServiceDescriptionImporter.GenerateWebReferences(__webReferences, __csharpCodeProvider, __compileUnit, __webReferenceOptions);

      // Make inheritance between the generated proxy client and the specified contract type associated with the specified wsdl document.
      foreach (CodeTypeDeclaration codeTypeDeclaration in __namespace.Types)
      {
        if ((codeTypeDeclaration.BaseTypes.Count > 0) && (codeTypeDeclaration.BaseTypes[0].BaseType == typeof(SoapHttpClientProtocol).FullName))
        {
          // Set the base type associated to the specified web service technology.
          codeTypeDeclaration.BaseTypes[0].BaseType = this.GetBaseType(webServiceTechnologies).FullName;

          // Find the service type that is associated with the specified contract interface type.
          // HACK: The behaviour depends on the choice made for the protocol.
          // HACK : ProtocolName = "Soap".
          WebServiceAttribute __webServiceAttribute1 = CodeDomHelper.GetWebServiceAttribute(codeTypeDeclaration);
          // HACK: The binding can be defined from a configuration file (Endpoint interface/wsdlocation/WebServiceName/WebServiceNamespace).
          WebServiceAttribute __webServiceAttribute2 = CodeDomHelper.GetWebServiceAttribute(_contractType);
          // If the contract interface type doesn't declare any web service attribute, all generated proxy will have to implement this contract.
          if ((__webServiceAttribute1 != null) && ((__webServiceAttribute2 == null) || (CodeDomHelper.Compare(__webServiceAttribute1, __webServiceAttribute2) == 0)))
          {
            codeTypeDeclaration.BaseTypes.Add(_contractType);
          }
        }
      }

      // Generate the client proxy code from the built CodeDOM tree.
      StringWriter __sourceCodeStringWriter = new StringWriter(CultureInfo.CurrentCulture);
      __csharpCodeProvider.GenerateCodeFromCompileUnit(__compileUnit, __sourceCodeStringWriter, new CodeGeneratorOptions());
      __sourceCodeStringWriter.Close();

      // Defines compiler parameters such as referenced assemblies.
      CompilerParameters __compilerParameters = new CompilerParameters();
      __compilerParameters.ReferencedAssemblies.Add("System.dll");
      __compilerParameters.ReferencedAssemblies.Add("System.Xml.dll");
      __compilerParameters.ReferencedAssemblies.Add("System.Web.Services.dll");
      __compilerParameters.ReferencedAssemblies.Add("System.Data.dll");
      // Reference the Wse 3.0 assembly if it is installed.
      Assembly __assembly = Assembly.Load(Wse.ASSEMBLY);
      if (__assembly != null)
      {
        __compilerParameters.ReferencedAssemblies.Add(__assembly.Location);
      }
      // To compile, you must refer the assembly that contains the contract interface type.
      __compilerParameters.ReferencedAssemblies.Add(Assembly.GetExecutingAssembly().Location);
      this.ReferenceAssemblies(__compilerParameters, _contractType.Assembly);

      __compilerParameters.GenerateExecutable = false;
      __compilerParameters.GenerateInMemory = false;
      __compilerParameters.IncludeDebugInformation = false;
      __compilerParameters.TempFiles = new TempFileCollection(Path.GetTempPath());

      // Compile the client proxy code.
      CompilerResults __compilerResults = __csharpCodeProvider.CompileAssemblyFromSource(__compilerParameters, __sourceCodeStringWriter.ToString());

      // TODO : Manage specifically error "CS0535" ?
      if (__compilerResults.Errors.Count > 0)
      {
        throw new DynamicCompilationException(__compilerResults);
      }

      // Rename temporary client proxy assembly in order to cache it for later use
      CompiledAssemblyCacheHelper.RenameTempAssembly(__compilerResults.PathToAssembly, _wsdlUri.AbsoluteUri);
      return __compilerResults.CompiledAssembly;
    }
    /// <summary>
    /// Download the WSDL document in a temporary folder.
    /// </summary>
    /// <returns>True if the WSDL is already in cache, otherwise false.</returns>
    /// <remarks>Operation 2</remarks>
    private bool DownloadWSDL()
    {
      AdvancedWebClient __webClient = new AdvancedWebClient();
      if (this.Credentials != null)
      {
        __webClient.Credentials = this.Credentials;
      }
      if (this.Proxy != null)
      {
        __webClient.Proxy = this.Proxy;
      }
      // Use as well as possible the request cache.
      __webClient.CachePolicy = new RequestCachePolicy(RequestCacheLevel.Revalidate);
      __webClient.DownloadFile(_wsdlUri, this.WsdlTemporaryFilePath);
      return __webClient.IsFromCache;
    }
    /// <summary>
    /// Builds the web service proxy assembly using specified builder options.
    /// </summary>
    /// <param name="options">The builder options.</param>
    /// <remarks>Operation 1</remarks>
    internal void Build(WebServiceProxyBuilderOptions options)
    {
      //Apply specified cache policy.
      switch (options.CachePolicy)
      {
        case CachePolicy.ClearAllCachedAssemblies:
          CompiledAssemblyCacheHelper.ClearAllCached();
          this.DownloadWSDL();
          _generatedAssembly = this.Generate(options.WebServiceTechnologies, options.ProtocolName);
          return;
        case CachePolicy.ClearMatchingCachedAssembly:
          CompiledAssemblyCacheHelper.ClearCache(_wsdlUri.AbsoluteUri);
          this.DownloadWSDL();
          _generatedAssembly = this.Generate(options.WebServiceTechnologies, options.ProtocolName);
          return;
        case CachePolicy.LocalCacheOnly:
          //check cache first
          _generatedAssembly = CompiledAssemblyCacheHelper.CheckCacheForAssembly(_wsdlUri.AbsoluteUri);
          if (_generatedAssembly == null)
          {
            // TODO : Localize this exception message.
            throw new TemporaryCacheException("The proxy assembly for the specified WSDL document: " + _wsdlUri.AbsoluteUri + " has not been found in the cache.");
          }
          return;
        case CachePolicy.LocalCacheIfAvailable:
          //check cache first
          _generatedAssembly = CompiledAssemblyCacheHelper.CheckCacheForAssembly(_wsdlUri.AbsoluteUri);
          //if the assembly is not in the cache, generate it.
          if (_generatedAssembly == null)
          {
            this.DownloadWSDL();
            _generatedAssembly = this.Generate(options.WebServiceTechnologies, options.ProtocolName);
          }
          return;
        case CachePolicy.CheckWsdlForInvalidation:
          //check the validity of the WSDL document.
          if (this.DownloadWSDL())
          {
            //check cache
            _generatedAssembly = CompiledAssemblyCacheHelper.CheckCacheForAssembly(_wsdlUri.AbsoluteUri);
          }
          //if the assembly is not in the cache, generate it.
          if (_generatedAssembly == null)
          {
            _generatedAssembly = this.Generate(options.WebServiceTechnologies, options.ProtocolName);
          }
          return;
      }
    }
    /// <summary>
    /// Builds the web service proxy assembly.
    /// </summary>
    /// <remarks>Operation 1</remarks>
    internal void Build()
    {
      this.Build(new WebServiceProxyBuilderOptions());
    }
    /// <summary>
    /// Get the generated proxy.
    /// </summary>
    /// <returns>The generated proxy.</returns>
    /// <remarks>Operation 5</remarks>
    internal SoapHttpClientProtocol GetResult()
    {
      if (_generatedAssembly == null)
      {
        throw new InvalidOperationException();
      }
      try
      {
        Type __proxyType = null;
        foreach (Type type in _generatedAssembly.GetTypes())
        {
          if (type.GetInterface(_contractType.FullName, false) != null)
          {
            __proxyType = type;
            break;
          }
        }
        SoapHttpClientProtocol __soapHttpClientProtocol = (SoapHttpClientProtocol)Activator.CreateInstance(__proxyType);
        // Bind the credentials and proxy properties with specified credentials and property values.
        if (this.Credentials != null)
        {
          __soapHttpClientProtocol.Credentials = this.Credentials;
        }
        if (this.Proxy != null)
        {
          __soapHttpClientProtocol.Proxy = this.Proxy;
        }
        return __soapHttpClientProtocol;
      }
      catch (Exception ex)
      {
        throw new ProxyTypeInstantiationException("An error occured while instantiating the proxy type: " + ex.Message + ", " + ex.StackTrace, ex);
      }
    }

    #endregion
  }
}