#region (c) 2002-2006 by thinktecture
// **********************************************************************
//
//  2002-2006 by thinktecture, Ingo Rammer and Christian Weyer. All rights reserved.
//
// This copy of thinktecture is licensed to you under the terms described in the
// License.txt file included in this distribution.
//
// Author: 
// Christian Weyer (christian.weyer@thinktecture.com)
//
// **********************************************************************
#endregion
#region (c) 2006 by netfxfactory.org
// **********************************************************************
//
//  2006 by netfxfactory.org. All rights reserved.
//
// This copy is licensed to you under the terms described in the
// License.txt file included in this distribution.
//
// Author: 
// pascal.recchia@netfxfactory.org
// anthony.guerot@netfxfactory.org
// **********************************************************************
#endregion

using System;
using System.Collections.Generic;
using System.Text;
using System.Web.Services.Description;
using System.Xml;
using System.Web.Services.Protocols;
using System.Net;
using System.Net.Cache;
using System.Reflection;

namespace thinktecture.ProxyModel.Web.Services
{
  /// <summary>
  /// Represents the web service communication platform (Asmx, Wse 3.0).
  /// </summary>
  /// <remarks>If the Wse 3.0 Api is installed, it will be this technology which will be used to communicate with web services.</remarks>
  internal sealed class WebServicesCommunicationPlatform : BaseCommunicationPlatform
  {
    #region fields

    private SoapHttpClientProtocol _soapHttpClientProtocol = null;
    private WebServiceTechnologies _webServiceTechnologies;

    #endregion
    #region ctors
    public WebServicesCommunicationPlatform()
    {
      // Determines what the web service technology will be used.
      _webServiceTechnologies = (Wse.IsInstalled() ? WebServiceTechnologies.Wse : WebServiceTechnologies.Asmx);
    }
    #endregion
    #region props

    // HACK : Theses properties must be setted from the relevant interface : ISoapHttpClientProtocol...
    //private IWebProxy Proxy
    //{
    //  get { return (IWebProxy)this.GetValue(WebServicesPropertyMetadata.ProxyProperty); }
    //  set { this.SetValue(WebServicesPropertyMetadata.ProxyProperty, value); }
    //}
    //private SoapProtocolVersion SoapVersion
    //{
    //  get { return (SoapProtocolVersion)this.GetValue(WebServicesPropertyMetadata.SoapVersionProperty); }
    //  set { this.SetValue(WebServicesPropertyMetadata.SoapVersionProperty, value); }
    //}
    //private int Timeout
    //{
    //  get { return (int)this.GetValue(WebServicesPropertyMetadata.TimeoutProperty); }
    //  set { this.SetValue(WebServicesPropertyMetadata.TimeoutProperty, value); }
    //}
    private IWebProxy Proxy
    {
      get { return (IWebProxy)this.GetValue(WebServicesPropertyMetadata.ProxyProperty); }
      set { this.SetValue(WebServicesPropertyMetadata.ProxyProperty, value); }
    }
    private ICredentials Credentials
    {
      get { return (ICredentials)this.GetValue(WebServicesPropertyMetadata.CredentialsProperty); }
      set { this.SetValue(WebServicesPropertyMetadata.CredentialsProperty, value); }
    }
    private CachePolicy CachePolicy
    {
      get { return (CachePolicy)this.GetValue(WebServicesPropertyMetadata.CachePolicyProperty); }
      set { this.SetValue(WebServicesPropertyMetadata.CachePolicyProperty, value); }
    }
    private string ProtocolName
    {
      get { return (string)this.GetValue(WebServicesPropertyMetadata.ProtocolNameProperty); }
      set { this.SetValue(WebServicesPropertyMetadata.ProtocolNameProperty, value); }
    }
    public override CommunicationTechnology CommunicationTechnology
    {
      get { return CommunicationTechnology.WebServices; }
    }

    #endregion
    #region methods

    protected override object GetBoundValue(DependencyProperty dependencyProperty)
    {
      // HACK : Theses properties must be setted from the relevant interface : ISoapHttpClientProtocol...
      //if (dependencyProperty.Name == WebServicesPropertyMetadata.ProxyProperty.Name)
      //{
      //  if (_soapHttpClientProtocol != null)
      //  {
      //    return _soapHttpClientProtocol.Proxy;
      //  }
      //}
      //else if (dependencyProperty.Name == WebServicesPropertyMetadata.SoapVersionProperty.Name)
      //{
      //  if (_soapHttpClientProtocol != null)
      //  {
      //    return _soapHttpClientProtocol.SoapVersion;
      //  }
      //}
      //else if (dependencyProperty.Name == WebServicesPropertyMetadata.TimeoutProperty.Name)
      //{
      //  if (_soapHttpClientProtocol != null)
      //  {
      //    return _soapHttpClientProtocol.Timeout;
      //  }
      //}
      //else if (dependencyProperty.Name == WebServicesPropertyMetadata.CredentialsProperty.Name)
      //{
      //  if (_soapHttpClientProtocol != null)
      //  {
      //    return _soapHttpClientProtocol.Credentials;
      //  }
      //}
      return null;
    }
    protected override void SetBoundValue(DependencyProperty dependencyProperty, object value)
    {
      // HACK : Theses properties must be setted from the relevant interface : ISoapHttpClientProtocol...
      //if (dependencyProperty.Name == WebServicesPropertyMetadata.ProxyProperty.Name)
      //{
      //  if (_soapHttpClientProtocol != null)
      //  {
      //    _soapHttpClientProtocol.Proxy = (IWebProxy)value;
      //  }
      //}
      //else if (dependencyProperty.Name == WebServicesPropertyMetadata.SoapVersionProperty.Name)
      //{
      //  if (_soapHttpClientProtocol != null)
      //  {
      //    _soapHttpClientProtocol.SoapVersion = (SoapProtocolVersion)value;
      //  }
      //}
      //else if (dependencyProperty.Name == WebServicesPropertyMetadata.TimeoutProperty.Name)
      //{
      //  if (_soapHttpClientProtocol != null)
      //  {
      //    _soapHttpClientProtocol.Timeout = (int)value;
      //  }
      //}
      //else if (dependencyProperty.Name == WebServicesPropertyMetadata.CredentialsProperty.Name)
      //{
      //  if (_soapHttpClientProtocol != null)
      //  {
      //    _soapHttpClientProtocol.Credentials = (ICredentials)value;
      //  }
      //}
    }
    public override bool IsDefined(Type contractType, Uri uri)
    {
      // TODO : Support custom configuration section to specify uri mapped with a service contract type.
      if (uri == null)
      {
        return false;
      }
      bool __isDefined = false;
      using (XmlReader __xmlReader = new XmlTextReader(uri.AbsoluteUri))
      {
        try
        {
          __isDefined = ServiceDescription.CanRead(__xmlReader);
        }
        //catch (WebException webException)
        //{
        //  // TODO : Throw an exception ?
        //  //throw webException;
        //  return false;
        //}
        catch (XmlException)
        {
          // TODO : Throw an exception ?
          return false;
        }
      }
      return __isDefined;
    }
    public override object GetProxy(Type contractType, Uri uri)
    {
      // Initialize a new instance of the Web Service Proxy builder.
      WebServiceProxyBuilder __webServiceProxyBuilder = new WebServiceProxyBuilder(uri.AbsoluteUri, contractType);
      // Set the credentials and proxy properties which will be used to get meta information on the web service with specified uri.
      __webServiceProxyBuilder.Credentials = this.Credentials;
      __webServiceProxyBuilder.Proxy = this.Proxy;
      // Build the proxy.
      __webServiceProxyBuilder.Build(new WebServiceProxyBuilderOptions(this.CachePolicy, _webServiceTechnologies, this.ProtocolName));

      // Get the built SoapHttpClientProtocol instance.
      return __webServiceProxyBuilder.GetResult();
      //SoapHttpClientProtocol __soapHttpClientProtocol = __webServiceProxyBuilder.GetResult();

      // HACK : Theses properties must be setted from the relevant interface : ISoapHttpClientProtocol...
      //// Set the properties of the built instance.
      //__soapHttpClientProtocol.Proxy = this.Proxy;
      //__soapHttpClientProtocol.Credentials = this.Credentials;
      //// TODO : Evaluate the link between this property and the field protocolName 
      ////__soapHttpClientProtocol.SoapVersion = this.SoapVersion;
      //if (this.Timeout != 0)
      //{
      //  __soapHttpClientProtocol.Timeout = this.Timeout;
      //}
      //_soapHttpClientProtocol = __soapHttpClientProtocol;
      //return _soapHttpClientProtocol;
    }

    #endregion
  }
}
