#region (c) 2002-2006 by thinktecture
// **********************************************************************
//
//  2002-2006 by thinktecture, Ingo Rammer and Christian Weyer. All rights reserved.
//
// This copy of thinktecture is licensed to you under the terms described in the
// License.txt file included in this distribution.
//
// Author: 
// Christian Weyer (christian.weyer@thinktecture.com)
//
// **********************************************************************
#endregion
#region (c) 2006 by netfxfactory.org
// **********************************************************************
//
//  2006 by netfxfactory.org. All rights reserved.
//
// This copy is licensed to you under the terms described in the
// License.txt file included in this distribution.
//
// Author: 
// pascal.recchia@netfxfactory.org
// anthony.guerot@netfxfactory.org
// **********************************************************************
#endregion

using System;
using System.Collections.Generic;
using System.Text;
using System.Reflection;

namespace thinktecture.ProxyModel.Web.Services
{
  /// <summary>
  /// Lists constants associated to the use of the Wse 3.0 Api.
  /// </summary>
  internal static class Wse
  {
    #region consts

    /// <summary>
    /// The long form of the assembly name associated to the Wse 3.0 Api.
    /// </summary>
    internal const string ASSEMBLY = "Microsoft.Web.Services3, Version=3.0.0.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35";

    ///// <summary>
    ///// The manifest module name of the assembly name associated to the Wse 3.0 Api.
    ///// </summary>
    //internal const string ASSEMBLY_MANIFESTMODULE_NAME = "Microsoft.Web.Services3.dll";

    /// <summary>
    /// The full name of the custom <see cref="SoapHttpClientProtocol"/> redefined by the Wse 3.0 Api.
    /// </summary>
    internal const string BASETYPE = "Microsoft.Web.Services3.WebServicesClientProtocol";

    #endregion
    #region methods

    /// <summary>
    /// Check if the Wse Api 3.0 is installed.
    /// </summary>
    /// <returns>True if the Wse Api 3.0 is installed, otherwise false.</returns>
    internal static bool IsInstalled()
    {
      Assembly __assembly = Assembly.Load(Wse.ASSEMBLY);
      return (__assembly != null);
    }
    /// <summary>
    /// Get the custom <see cref="SoapHttpClientProtocol"/> type redefined by the Wse 3.0 Api.
    /// </summary>
    /// <returns>Null if the Wse Api 3.0 is not installed on the machine, otherwise the custom <see cref="SoapHttpClientProtocol"/> type redefined by the Wse 3.0 Api.</returns>
    /// <exception cref="InvalidOperationException">The Wse 3.0 Api is not correctly configured on this machine.</exception>
    internal static Type GetWebServicesClientProtocolType()
    {
      Assembly __assembly = Assembly.Load(Wse.ASSEMBLY);
      if (__assembly == null)
      {
        // TODO : Localize this exception message.
        throw new InvalidOperationException("The Wse 3.0 Api is not correctly configured on this machine.");
      }
      Type __webServicesClientProtocolType = __assembly.GetType(Wse.BASETYPE);
      if (__webServicesClientProtocolType == null)
      {
        // TODO : Localize this exception message.
        throw new InvalidOperationException("The Wse 3.0 Api is not correctly configured on this machine.");
      }
      return __webServicesClientProtocolType;
    }

    #endregion
  }
}
