using System;
using System.IO;
using System.Collections.Generic;
using System.Web;
using System.Web.Services;
using Microsoft.Web.Services3.Messaging;
//
// StockService implements IStockService, thereby implementing the service contract 
// define on the interface definition
//
namespace StockServiceLibrary
{
    //
    // Use [WebService] to specify the service namespace, the remaining contract
    // details are specified by IStockService
    //
    //
    // The use of [SoapActor] allows for tracing intermediaries in this lab
    //
    [SoapActor("*")]
    [WebService(
        Namespace = "http://stockservice.contoso.com/wse/samples")]
    public class StockService : IStockService
    {
        public StockQuote[] GetStockQuotes(string[] symbols)
        {
            //
            // Random stock quote implementation
            //
            List<StockQuote> quotes = new List<StockQuote>();

            Random rnd = new Random();

            foreach (String symbol in symbols)
            {
                StockQuote quote = new StockQuote();
                double price = rnd.NextDouble() * 100;
                quote.Symbol = symbol;
                quote.Date = DateTime.Now;
                quote.Last = price;
                quote.Change = rnd.NextDouble() * 10;
                quote.High = price + 10;
                quote.Low = (price - 10 > 0) ? price - 10 : 0;
                quotes.Add(quote);
            }
            return quotes.ToArray();
        }

        public byte[] GetChart(string symbol)
        {
            string fileName;

            //
            // if HttpContext.Current is null, then we know we're not being hosted by ASP.NET
            //
            if (HttpContext.Current == null)
                fileName = @"..\..\chart.gif";
            else
                fileName = HttpContext.Current.Server.MapPath("chart.gif");

            // read the file and return the byte[]
            using (FileStream fs = new FileStream(fileName, FileMode.Open, FileAccess.Read, FileShare.Read))
            {
                byte[] buffer = new byte[fs.Length];
                fs.Read(buffer, 0, (int)fs.Length);
                return buffer;
            }
        }

    }
}