using System;
using System.IO;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using Microsoft.Web.Services3.Messaging;
using Microsoft.Web.Services3.Addressing;

using thinktecture.ProxyModel;

namespace WindowsClient
{
  public partial class Form1 : Form
  {
    public Form1()
    {
      InitializeComponent();
      this.dataGridView1.AutoGenerateColumns = true;
    }

    private void button1_Click(object sender, EventArgs e)
    {
      try
      {
        //
        // Extra the stock symbols entered in the form
        //
        string[] symbols = textBoxSymbols.Text.Split(' ');
        // 
        // Instantiate the StockService proxy (use the WSE version),
        // and specify the target Url
        //
        ProxyFactory<IStockServiceClient> __proxyFactory = new ProxyFactory<IStockServiceClient>(new Uri(comboBoxAddress.Text));
        IStockServiceClient service = __proxyFactory.CreateProxy();

        //
        // Invoke GetStockQuotes and bind the returned StockQuote[]
        // to the datagrid's DataSource property
        //
        dataGridView1.DataSource = service.GetStockQuotes(symbols);
      }
      catch (Exception ex)
      {
        MessageBox.Show(ex.Message);
      }
    }

    private void buttonGetChart_Click(object sender, EventArgs e)
    {
      try
      {
        //
        // Ensure a symbol has been selected
        //
        if (dataGridView1.SelectedCells.Count == 0)
        {
          MessageBox.Show("You must select a symbol first");
          return;
        }
        // 
        // Instantiate the StockService proxy (use the WSE version),
        // and specify the target Url
        //
        ProxyFactory<IStockServiceClient> __proxyFactory = new ProxyFactory<IStockServiceClient>(new Uri(comboBoxAddress.Text));
        IStockServiceClient service = __proxyFactory.CreateProxy();

        //
        // Require MTOM in the request
        //
        service.RequireMtom = true;

        //
        // Invoke GetChart supplying the selected symbol
        //
        string symbol = (string)dataGridView1.SelectedCells[0].Value;
        byte[] chart = service.GetChart(symbol);
        //
        // Load the returned byte[] into a Bitmap and display
        //
        MemoryStream ms = new MemoryStream(chart);
        Bitmap bm = new Bitmap(ms);
        pictureBox1.Image = bm;
      }
      catch (Exception ex)
      {
        MessageBox.Show(ex.Message);
      }
    }
  }
}